Package: AnnotationBustR
Version: 1.3.0
Date: 2020-09-23
Title: Extract Subsequences from GenBank Annotations
Author: Samuel R. Borstein <sam@borstein.com>, Brian O'Meara <bomeara@utk.edu>
Maintainer: Samuel R. Borstein <sam@borstein.com>
Depends: R (>= 3.4)
Imports: ape (>= 4.1), seqinr (>= 3.3-6)
Description: Extraction of subsequences into FASTA files from GenBank annotations where gene names may vary among accessions. Borstein & O'Meara (2018) <doi:10.7717/peerj.5179>.
License: GPL (>= 2)
LazyData: true
RoxygenNote: 7.1.1
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
URL: https://github.com/sborstein/AnnotationBustR,
        https://www.ncbi.nlm.nih.gov/nuccore,
        https://peerj.com/articles/5179/
BugReports: https://github.com/sborstein/AnnotationBustR/issues
NeedsCompilation: no
Packaged: 2020-09-24 03:08:57 UTC; sam
Repository: CRAN
Date/Publication: 2020-09-24 08:10:13 UTC
Built: R 4.6.0; ; 2026-04-26 02:36:42 UTC; unix
